#include <Wire.h>
#include "rv3029.h"  // Needed to make the IDE accept our time struct.

#define I2C_ADDRESS  (0x56)

uint8_t bcd2bin(uint8_t val)
{
  return ((val) & 0x0f) + ((val) >> 4) * 10;
}

uint8_t bin2bcd(uint8_t val)
{
  return (((val / 10) << 4) | (val % 10));
}

void rtc_set_time(uint8_t i2c_address, rtc_time_t& rtc_time)
{
  Wire.beginTransmission(i2c_address);            
  Wire.write(0x08);  // Address of seconds register.
  Wire.write(bin2bcd(rtc_time.seconds));  // Set seconds
  Wire.write(bin2bcd(rtc_time.minutes));  // Set minutes
  Wire.write(bin2bcd(rtc_time.hours));  // Set hours
  Wire.write(bin2bcd(rtc_time.day_of_month));  // Set date
  Wire.write(bin2bcd(rtc_time.day_of_week));  // Set weeek day
  Wire.write(bin2bcd(rtc_time.month));  // Set month.
  Wire.write(bin2bcd(rtc_time.year));  // Set year.
  Wire.endTransmission();     
}

void rtc_read_time(uint8_t i2c_address, rtc_time_t& rtc_time)
{
  Wire.beginTransmission(i2c_address);
  Wire.write(0x08);  // Address of first time register (seconds).
  Wire.endTransmission();
  // Request the seven date/time bytes.
  Wire.requestFrom(i2c_address,(uint8_t)7); // Typecast required to avoid C++ warning.
  rtc_time.seconds = bcd2bin(Wire.read()&0x7f);
  rtc_time.minutes = bcd2bin(Wire.read()&0x7f);
  uint8_t hours = Wire.read(); // Hours need more processing.
  rtc_time.day_of_month = bcd2bin(Wire.read()&0x3f);
  rtc_time.day_of_week = bcd2bin(Wire.read()&0x07);
  rtc_time.month = bcd2bin(Wire.read()&0x1f);
  rtc_time.year = bcd2bin(Wire.read());

  // Convert hours.
  if ((hours&(1<<6))!=0)
  {
    // 12h format.
    rtc_time.hours = bcd2bin(hours&0x1f);
    // Check PM flag.
    if ((hours&(1<<5))!=0) rtc_time.hours += 12;
  }
  else rtc_time.hours = bcd2bin(hours&0x3f);
}

void print_time_value(uint16_t value)
{
  if (value<=9) Serial.print('0');
  Serial.print(value);
}

void print_time(rtc_time_t& rtc_time, boolean newline)
{
  print_time_value(rtc_time.hours);
  Serial.print(":");    
  print_time_value(rtc_time.minutes);
  Serial.print(":");    
  print_time_value(rtc_time.seconds);
  if (newline==true) Serial.println();
}

void print_date(rtc_time_t& rtc_time, boolean newline)
{
  print_time_value(rtc_time.day_of_month);
  Serial.print("/");    
  print_time_value(rtc_time.month);
  Serial.print("/");    
  print_time_value(rtc_time.year+2000);
  if (newline==true) Serial.println();
}

void setup(void) 
{
  rtc_time_t rtc_time;
  rtc_time.seconds = 0;
  rtc_time.minutes = 52;
  rtc_time.hours = 14;
  rtc_time.day_of_week = 2;
  rtc_time.day_of_month = 13;
  rtc_time.month = 10;
  rtc_time.year = 15;

  // This is because we connected the GND pin of the module to A2.
  pinMode(A2,OUTPUT);
  digitalWrite(A2,LOW);
  
  // Initialise communication ports.
  Serial.begin(9600);
  Wire.begin();

  // Setup RTC chip.
  Wire.beginTransmission(I2C_ADDRESS);          
  Wire.write(0x00);  // CONTROL_1 address.
  Wire.write(0x01);  // CONTROL_1: Enable 1 Hz clock source for watch.
  Wire.write(0x00);  // CONTROL_INT: Disable all interrupts.
  Wire.write(0x00);  // CONTROL_INT FLAG: Clear all.
  Wire.write(0x00);  // CONTROL_STATUS: Clear all, including PON.
  Wire.endTransmission();  
  rtc_set_time(I2C_ADDRESS,rtc_time);
}

void loop(void)
{
  rtc_time_t rtc_time;
  rtc_read_time(I2C_ADDRESS,rtc_time);
  print_time(rtc_time,false);
  Serial.print(" - ");    
  print_date(rtc_time,true);
  delay(1000);
}

